#!/bin/bash
#
# Author: Thomas Huth (THUTH@de.ibm.com)
# $Id$
#---------------------------------------------------------------------------------------------------

# Front-End Vmin Script version
FEVMINVERSION="1.7"


function showhelp
{
	echo "FrontEnd Vmin script version" $FEVMINVERSION
	echo "Usage:"
	echo " $0 <RC-IP> run [<node0> <node1> ...]      Run complete Vmin process"
	echo " $0 <RC-IP> runall [<node0> <node1> ...]   Force run complete Vmin"
	echo " $0 <RC-IP> start [<node0> <node1> ...]    Just start w/o waiting"
	echo " $0 <RC-IP> startall [<node0> <node1> ...] Force start w/o waiting"
	echo " $0 <RC-IP> disable [<node0> <node1> ...]  Disable MFG boot mode"
	echo " $0 <RC-IP> check [<node0> <node1> ...]    Check Vmin settings in VPD"
	echo " $0 <RC-IP> stats [<node0> <node1> ...]    Get statistics from VPD"
	echo " $0 <RC-IP> getgb [<node0> <node1> ...]    Readout guardband values"
	echo " $0 <RC-IP> setgb=<volt> [<node0> ...]     Set guardband value in VPD"
	echo
	echo "<RC-IP> is the IP address of the rootcard. "
	echo "Node numbers must be specified as hex values between 00 and 0f"
	echo "If no node numers are given, all 16 nodes are used."
}

if [ $# -lt 2 -o "$1" = "-h" -o "$1" = "--help" ]; then
	showhelp
	exit -1
fi

# Get the root card IP address (or corresponding hostname)
# e.g. RCIP=10.98.232.101
RCIP="$1"
shift

# Get the action that should be executed
ACTION="$1"
shift

# Get the node numbers
if  [ $# -lt 1 ]; then
	NODES="00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f"
else
	NODES=$*
fi


# Send command to a Service Processor
# Param $1 = node, Param $2... = SP command bytes
function feensp
{
	OUTPUT=`${QROOT}/bin/feentest -a $RCIP -t service $* 2> /dev/null`
	# Is status not OK?
	if echo "$OUTPUT" | grep -q "RS485_Timeout"; then
		# Timeout, service processor still busy
		echo "ERROR (TIMEOUT)"
	elif echo "$OUTPUT" | grep -q "STAT:  fa"; then
		# Service processor is currently busy
		echo "ERROR (BUSY)"
	elif echo "$OUTPUT" | grep -q -i "error"; then
		# Error from feentest
		echo "ERROR"
		echo >&2
		echo " feentest -a $RCIP -t service $*" >&2
		echo "$OUTPUT" >&2
		echo >&2
	elif ! echo "$OUTPUT" | grep -q "STAT:  00"; then
		# Error from service processor
		echo "ERROR"
		echo >&2
		echo " feentest -a $RCIP -t service $*" >&2
		echo "$OUTPUT" >&2
		echo >&2
	else
		# Command executed successfully
		OUTRET=`echo "$OUTPUT" | grep "DATA:" | sed "s/DATA: //"`
		if [ "$OUTRET" = "" ]; then
			echo "OK"
		else
			echo "$OUTRET"
		fi
	fi
}


function showconfig
{
	echo "FrontEnd Vmin script version" $FEVMINVERSION
	echo "RootCard IP:" $RCIP
	echo "Nodes:" $NODES
}

# Start Vmin on the nodes
# Param $1 = force|normal
function startvmin
{
	# Power-off
	for n in $NODES ; do
		echo -n "Node $n: Power-off... "
		ERRSTAT=`feensp $n 30 00`
		echo $ERRSTAT
		if echo "$ERRSTAT" | grep -qi "ERROR"; then
			echo "Please fix above error first, then restart!"
			exit -1
		fi
	done

	# VPD-Write to set MFG boot mode:
	for n in $NODES ; do
		echo -n "Node $n: Set MFG boot... "
		ERRSTAT=`feensp $n 32 00 14 01 03`
		echo $ERRSTAT
		if echo "$ERRSTAT" | grep -qi "ERROR"; then
			exit -1
		fi
	done

	# VPD-Write to destroy Vmin magic:
	if [ "$1" = "force" ]; then
		for n in $NODES ; do
			echo -n "Node $n: Erase Vmin magic... "
			ERRSTAT=`feensp $n 32 70 60 04 de ad ca fe`
			echo $ERRSTAT
			if echo "$ERRSTAT" | grep -qi "ERROR"; then
				exit -1
			fi
		done
	fi

	# Power-on
	for n in $NODES ; do
		echo -n "Node $n: Power-on... "
		ERRSTAT=`feensp $n 30 01`
		echo $ERRSTAT
		if echo "$ERRSTAT" | grep -qi "ERROR"; then
			exit -1
		fi
	done
}


# Wait until all SPs of the node cards answer to requests on the RS485 bus again.
# Time out after 22 minutes
function pollforcompletion
{
	NOTDONEYET=$NODES
	let ticks=0
	starttime=`date '+%s'`
	while [ "${NOTDONEYET// /}" != "" -a $ticks -lt 1320 ]; do
		# echo "Not done yet: '${NOTDONEYET}'"
		for n in $NOTDONEYET ; do
			echo -n "Node $n: "
			MAGIC=`feensp $n 31 70 60 04`
			if ! echo "$MAGIC" | grep -q "ERROR"; then
				echo "Finished."
				NOTDONEYET=${NOTDONEYET/$n/}
			else
				echo "Still running."
			fi
		done
		let ticks=`date '+%s'`-$starttime
	done

	if [ "${NOTDONEYET// /}" != "" ]; then
		echo "ERROR : Nodes did not finish in time:" $NOTDONEYET
		for n in $NOTDONEYET ; do
			NODES=${NODES/$n/}
		done
	fi
}


# VPD-Write to disable MFG boot mode:
function disablevmin
{
	for n in $NODES ; do
		echo -n "Node $n: Setting normal boot mode... "
		feensp $n 32 00 14 01 02
	done
}


# VPD-read to check if Vmin section has been written correctly:
function checkvmin
{
	for n in $NODES ; do
		echo -n "Node $n: Vmin result: "
		MAGIC=`feensp $n 31 70 60 04`
		if ! echo "$MAGIC" | grep -q "4d 69 6e 01"; then
			echo "FAILED! ERROR: WRONG MAGIC: " $MAGIC
		else
			ERRCODE=`feensp $n 31 70 7e 02`
			if ! echo "$ERRCODE" | grep -q "00 00"; then
				echo "FAILED! ERROR: " $ERRCODE
			else
				echo -n "SUCCESS! Vmin voltage: "
				let VOLT=`feensp $n 31 70 7c 02 | awk '{ print "0x"$1$2 }'`
				echo $VOLT mV
			fi
		fi
	done
}


# Read out statistics information from VPD:
function stats
{
	for n in $NODES ; do
		echo -n "Node $n: "
		MAGIC=`feensp $n 31 70 60 04`
		if ! echo "$MAGIC" | grep -q "4d 69 6e 01"; then
			echo "ERROR: WRONG MAGIC: " $MAGIC
		else
			echo -n " Vfused: "
			ERRCODE=`feensp $n 31 70 72 02`
			if ! echo "$ERRCODE" | grep -q "00 00"; then
				echo "ERROR: " $ERRCODE
			else
				let VOLTSET=`feensp $n 31 70 70 02 | awk '{ print "0x"$1$2 }'`
				let TEMP=`   feensp $n 31 70 74 02 | awk '{ print "0x"$1$2 }'`
				let VOLTGET=`feensp $n 31 70 76 02 | awk '{ print "0x"$1$2 }'`
				let AMPS=10*`feensp $n 31 70 78 02 | awk '{ print "0x"$1$2 }'`
				echo -n -e "$VOLTSET mV, $VOLTGET mV, $AMPS mA, $TEMP C\t"
			fi

			echo -n " Vmin: "
			ERRCODE=`feensp $n 31 70 7e 02`
			if ! echo "$ERRCODE" | grep -q "00 00"; then
				echo "ERROR: " $ERRCODE
			else
				let VOLTSET=`feensp $n 31 70 7c 02 | awk '{ print "0x"$1$2 }'`
				let TEMP=`   feensp $n 31 70 80 02 | awk '{ print "0x"$1$2 }'`
				let VOLTGET=`feensp $n 31 70 82 02 | awk '{ print "0x"$1$2 }'`
				let AMPS=10*`feensp $n 31 70 84 02 | awk '{ print "0x"$1$2 }'`
				echo -n -e "$VOLTSET mV, $VOLTGET mV, $AMPS mA, $TEMP C\t"
			fi

			echo -n " Vmin+GB: "
			ERRCODE=`feensp $n 31 70 8a 02`
			if ! echo "$ERRCODE" | grep -q "00 00"; then
				echo "ERROR: " $ERRCODE
			else
				let VOLTSET=`feensp $n 31 70 88 02 | awk '{ print "0x"$1$2 }'`
				let TEMP=`   feensp $n 31 70 8c 02 | awk '{ print "0x"$1$2 }'`
				let VOLTGET=`feensp $n 31 70 8e 02 | awk '{ print "0x"$1$2 }'`
				let AMPS=10*`feensp $n 31 70 90 02 | awk '{ print "0x"$1$2 }'`
				echo "$VOLTSET mV, $VOLTGET mV, $AMPS mA, $TEMP C"
			fi
		fi
	done
}


# VPD-read to read guardband value from VPD:
function getgb
{
	for n in $NODES ; do
		echo -n "Node $n: Guardband: "
		let GBVOLT=`feensp $n 31 70 6f 01 | awk '{ print "0x"$1 }'`
		if [ "$GBVOLT" = "0" ]; then
			echo "69 mV (default)"
		else
			echo $GBVOLT mV
		fi
	done
}

# VPD-write to set guardband value in VPD:
function setgb
{
	GBX=`echo $1 | awk '{ printf("%x", $1); }'`
	for n in $NODES ; do
		echo -n "Node $n: Setting guardband... "
		feensp $n 32 70 6f 01 $GBX
	done
}


case "$ACTION" in
	run|runall)
		showconfig
		if [ "$ACTION" = "runall" ]; then
			startvmin force
		else
			startvmin normal
		fi
		echo -n "Now waiting 2 minutes... "
		for i in 1 0 ; do sleep 60 ; echo -n "$i " ; done
		echo
		echo "Polling nodes until all are finished..."
		pollforcompletion
		disablevmin
		checkvmin
		if [ "${NOTDONEYET// /}" != "" ]; then
			echo "ERROR : Some nodes did not finish in time:" $NOTDONEYET
		fi
		;;
	start|startall)
		showconfig
		if [ "$ACTION" = "startall" ]; then
			startvmin force
		else
			startvmin normal
		fi
		;;
	disable)
		showconfig
		disablevmin
		;;
	check)
		checkvmin
		;;
	stats)
		stats
		;;
	poll)
		showconfig
		pollforcompletion
		;;
	getgb)
		getgb
		;;
	setgb=*)
		showconfig
		echo "Setting guardband to ${ACTION/setgb=/} mV."
		setgb ${ACTION/setgb=/}
		;;
	*)
		showhelp
esac
